@@ -18,6 +18,8 @@ try:
 except ImportError:
     import json
 
+from .dimensions import build_dimension
+
 
 class Filter:
     def __init__(self, **args):
@@ -56,6 +58,13 @@ class Filter:
                                       "upper": args["upper"],
                                       "upperStrict": args["upperStrict"],
                                       "alphaNumeric": args["alphaNumeric"]}}
+        elif args["type"] == "columnComparison":
+            self.filter = {"filter": {"type": "columnComparison",
+                                      "dimensions": args["dimensions"]}}
+        elif args["type"] == "interval":
+            self.filter = {"filter": {"type": "interval",
+                                      "dimension": args["dimension"],
+                                      "intervals": args["intervals"]}}
         else:
             raise NotImplementedError(
                 'Filter type: {0} does not exist'.format(args['type']))
@@ -91,6 +100,9 @@ class Filter:
         elif filter['type'] in ['not']:
             filter = filter.copy()
             filter['field'] = Filter.build_filter(filter['field'])
+        elif filter['type'] in ['columnComparison']:
+            filter = filter.copy()
+            filter['dimensions'] = [build_dimension(d) for d in filter['dimensions']]
 
         return filter
 
@@ -116,7 +128,8 @@ class JavaScript:
 
 class Bound(Filter):
     """
-    Bound filter can be used to filter by comparing dimension values to an upper value or/and a lower value. 
+    Bound filter can be used to filter by comparing dimension values to an
+    upper value or/and a lower value.
 
     :ivar str dimension: Dimension to filter on.
     :ivar str lower: Lower bound.
@@ -125,10 +138,28 @@ class Bound(Filter):
     :ivar bool upperStrict: Strict upper inclusion. Initial value: False
     :ivar bool alphaNumeric: Numeric comparison. Initial value: False
     """
-    def __init__(self, dimension, lower, upper, lowerStrict=False, upperStrict=False, alphaNumeric=False):
-
-        Filter.__init__(self,
+    def __init__(
+            self, dimension, lower, upper, lowerStrict=False,
+            upperStrict=False, alphaNumeric=False):
+        Filter.__init__(
+            self,
             type='bound', dimension=dimension,
             lower=lower, upper=upper,
             lowerStrict=lowerStrict, upperStrict=upperStrict,
             alphaNumeric=alphaNumeric)
+
+
+class Interval(Filter):
+    """
+    Interval filter can be used to filter by comparing dimension(__time)
+    values to a list of intervals.
+
+    :ivar str dimension: Dimension to filter on.
+    :ivar list intervals: List of ISO-8601 intervals of data to filter out.
+    """
+    def __init__(self, dimension, intervals):
+
+        Filter.__init__(
+            self,
+            type='interval', dimension=dimension,
+            intervals=intervals)
